unit MTEApi;

interface

uses
  Windows;
  
const
  //  ,   MTExxxx
  MTE_OK             = 0;
  MTE_CONFIG         = -1;
  MTE_SRVUNAVAIL     = -2;
  MTE_LOGERROR       = -3;
  MTE_INVALIDCONNECT = -4;
  MTE_NOTCONNECTED   = -5;
  MTE_WRITE          = -6;
  MTE_READ           = -7;
  MTE_TSMR           = -8;
  MTE_NOMEMORY       = -9;
  MTE_ZLIB           = -10;
  MTE_PKTINPROGRESS  = -11;
  MTE_PKTNOTSTARTED  = -12;
  MTE_FATALERROR		 = -13; // new CMA error code, previously was MTE_LOGON for TEServer
  MTE_INVALIDHANDLE  = -14;
  MTE_DSROFF         = -15;
  MTE_UNKNOWN        = -16;
  MTE_BADPTR         = -17;
  MTE_WRONGPARAM		 = -17; // CMA synonim for MTE_BADPTR
  MTE_TRANSREJECTED  = -18;
  MTE_REJECTION		   = -18; // CMA synonim for MTE_TRANSREJECTED
  // CMA error codes
  MTE_TEUNAVAIL		   = -19; // new CMA error code, previously was MTE_TOOSLOWCONNECT for TEServer
  MTE_NOTLOGGEDIN    = -20; // new CMA error code, previously was MTE_CRYPTO_ERROR for TEServer
  MTE_WRONGVERSION   = -21; // new CMA error code, previously was MTE_THREAD_ERROR for TEServer
  // TEServer errors again
  MTE_LOGON          = -30; // previously was -13, changed for CMA compatibility
  MTE_TOOSLOWCONNECT = -31; // previously was -19, changed for CMA compatibility
  MTE_CRYPTO_ERROR   = -32; // previously was -20, changed for CMA compatibility
  MTE_THREAD_ERROR   = -33; // previously was -21, changed for CMA compatibility
  MTE_NOTIMPLEMENTED = -34;

  //     
  ffKey       = $01;              //  
  ffSecCode   = $02;              //    
  ffNotNull   = $04;              //     
  ffVarBlock  = $08;              //     ,     

  ffFixedMask = $30;
  ffFixed1    = $10;              //     fiFixed: Decimals   1,   2
  ffFixed3    = $20;              //     fiFixed: Decimals   3,   2
  ffFixed4    = $30;              //     fiFixed: Decimals   4,   2

  //  
  tfUpdateable    = $01;          // 
  tfClearOnUpdate = $02;          //   
  tfOrderBook     = $04;          //   ""

type
  PMTEMsg = ^TMTEMsg;
  TMTEMsg = record
    DataLen: Cardinal;
    Data: record end;
  end;

  PMTERow = ^TMTERow;
  TMTERow = packed record
    FldCount: Byte;
    RowLen: Integer;
    RowData: record end;
  end;

  PMTETable = ^TMTETable;
  TMTETable = record
    Ref: Integer;
    RowCount: Integer;
    TblData: TMTERow;
  end;

  PMTETables = ^TMTETables;
  TMTETables = record
    TblCount: Integer;
    Tables: TMTETable;
  end;

  TErrorMsg = array [0..255] of Char;

  TTEFieldType = (ftChar, ftInteger, ftFixed, ftFloat, ftDate, ftTime);
  TTEEnumKind = (ekCheck, ekGroup, ekCombo);

const
  // connection property flags
  ZLIB_COMPRESSED = 1;
  FLAG_ENCRYPTED  = 2;
  FLAG_SIGNING_ON = 4;

  // certificate types
  CERTTYPE_UNKNOWN  = 0;
  CERTTYPE_VALIDATA = 1;
  CERTTYPE_OPENSSL  = 2;
  
type
  TMTECertAltName = packed record
    Subject: PChar;
    OrganizationName: PChar;        {*<   (WIN1251). }
    Surname: PChar;                 {*< ..   (WIN1251). }
    Description: PChar;             {*< . }
  end;

  TMemBlk =  record
    len: Longint;              {*<   }
    buf: Pointer;              {*<  }
  end;
  PMemBlk = ^TMemBlk;

  PMTEConnCertificate = ^TMTEConnCertificate;
  TMTEConnCertificate = packed record
    Owner,
    Issuer: TMTECertAltName;
    ValidFrom,
    ValidTo: TSystemTime;
    PrivateValidFrom: TSystemTime;
    PrivateValidTo: TSystemTime;
    CertType: Integer;    // one of CERTTYPE_xxxx constants
    CertExpired: Integer; // 0 - valid certificate, 1 - certificate or private key expired
    CertRevoked: Integer; // 0 - valid certificate, 1 - certificate revoked
    CertEncoded: TMemBlk; // certificate data in DER-format
  end;

  PMTEConnStats = ^TMTEConnStats;
  TMTEConnStats = packed record
    Size: Integer;
    Properties: DWORD;
    SentPackets: DWORD;
    RecvPackets: DWORD;
    SentBytes: DWORD;
    RecvBytes: DWORD;
    // fields added in version 2
    ServerIpAddress: DWORD;
    ReconnectCount: Integer;
    SentUncompressed: DWORD;
    RecvUncompressed: DWORD;
    ServerName: array [0..63] of Char;
    // fields added in version 3
    TsmrPacketSize: DWORD;
    TsmrSent: DWORD;
    TsmrRecv: DWORD;
  end;

  TMTEConnStats_v2 = packed record
    Size: Integer;
    Properties: DWORD;
    SentPackets: DWORD;
    RecvPackets: DWORD;
    SentBytes: DWORD;
    RecvBytes: DWORD;
    // fields added in version 2
    ServerIpAddress: DWORD;
    ReconnectCount: Integer;
    SentUncompressed: DWORD;
    RecvUncompressed: DWORD;
    ServerName: array [0..63] of Char;
  end;

  TMTEConnStats_v1 = packed record
    Size: Integer;
    Properties: DWORD;
    SentPackets: DWORD;
    RecvPackets: DWORD;
    SentBytes: DWORD;
    RecvBytes: DWORD;
  end;

  PMTEServInfo = ^TMTEServInfo;
  TMTEServInfo = packed record
    Connected_To_MICEX: Integer;
    Session_Id: Integer;
    MICEX_Sever_Name: array [0..32] of Char;
    Version_Major: Byte;
    Version_Minor: Byte;
    Version_Build: Byte;
    Beta_version: Byte;
    Debug_flag: Byte;
    Test_flag: Byte;
    Start_Time: Integer;
    Stop_Time_Min: Integer;
    Stop_Time_Max: Integer;
    Next_Event: Integer;
    Event_Date: Integer;
    Boards: record end;
  end;
  Service_Info = TMTEServInfo;

function MTEConnect(Params, ErrorMsg: LPSTR): Integer; stdcall;
function MTEDisconnect(Idx: Integer): Integer; stdcall;
function MTEStructure(Idx: Integer; var Msg: PMTEMsg): Integer; stdcall;
function MTEExecTrans(Idx: Integer; TransName, Params, ResultMsg: LPSTR): Integer; stdcall;
function MTEOpenTable(Idx: Integer; TableName, Params: LPSTR; Complete: BOOL;
  var Msg: PMTEMsg): Integer; stdcall;
function MTECloseTable(Idx, HTable: Integer): Integer; stdcall;
function MTEAddTable(Idx, HTable, Ref: Integer): Integer; stdcall;
function MTERefresh(Idx: Integer; var Msg: PMTEMsg): Integer; stdcall;
function MTEErrorMsg(ErrCode: Integer): PChar; stdcall;
function MTEFreeBuffer(Idx: Integer): Integer; stdcall;
function MTEGetSnapshot(Idx: Integer; var Snapshot: PChar; var Len: Integer): Integer; stdcall;
function MTESetSnapshot(Idx: Integer; Snapshot: PChar; Len: Integer; ErrorMsg: LPSTR): Integer; stdcall;
// optional functions, depend on version of MTESRL.DLL
function MTEGetExtData(Idx: Integer; DataSetName, ExtFileName: LPSTR; var Msg: PMTEMsg): Integer; stdcall;
function MTEExecTransIP(Idx: Integer; TransName, Params, ResultMsg: LPSTR; ClientIP: Integer): Integer; stdcall;
function MTEConnectionStats(Idx: Integer; var Stats: TMTEConnStats): Integer; stdcall;
function MTEConnectionStatus(Idx: Integer): Integer; stdcall;
function MTEConnectionCerts(Idx: Integer; MyCert, ServerCert: PMTEConnCertificate): Integer; stdcall;
function MTEGetVersion: PChar; stdcall;
function MTEGetServInfo(Idx: Integer; var ServInfo: PChar; var Len: Integer): Integer; stdcall;
function MTESelectBoards(Idx: Integer; BoardList: LPSTR): Integer; stdcall;
// CMA compatibility functions
function MTESoftConnect(Params, ErrorMsg: LPSTR): Integer; stdcall;
function MTELogon(Idx: Integer; sync_time: Integer): Integer; stdcall;

type
  TMTESrlAddOn = (msaExecTransIP, msaGetExtData, msaConnStats, msaConnStatus,
    msaGetVersion, msaConnCerts, msaGetServInfo, msaSoftConnect, msaLogon,
    msaSelectBoards);
  TMTESrlAddOns = set of TMTESrlAddOn;

function GetMTESrlAddons: TMTESrlAddOns;

implementation

type
  TMTEExecTransIP = function (Idx: Integer; TransName, Params, ResultMsg: LPSTR; ClientIP: Integer): Integer; stdcall;
  TMTEGetExtData = function (Idx: Integer; DataSetName, ExtFileName: LPSTR; var Msg: PMTEMsg): Integer; stdcall;
  TMTEConnectionStats = function (Idx: Integer; var Stats: TMTEConnStats): Integer; stdcall;
  TMTEConnectionStatus = function (Idx: Integer): Integer; stdcall;
  TMTEGetVersion = function : PChar; stdcall;
  TMTEConneectionCerts = function (Idx: Integer; MyCert, ServerCert: PMTEConnCertificate): Integer; stdcall;
  TMTEGetServInfo = function (Idx: Integer; var ServInfo: PChar; var Len: Integer): Integer; stdcall;
  TMTESoftConnect = function (Params, ErrorMsg: LPSTR): Integer; stdcall;
  TMTELogon = function (Idx: Integer; sync_time: Integer): Integer; stdcall;
  TMTESelectBoards = function (Idx: Integer; BoardList: LPSTR): Integer; stdcall;

var
  _MTEExecTransIP: TMTEExecTransIP = nil;
  _MTEGetExtData: TMTEGetExtData = nil;
  _MTEConnectionStats: TMTEConnectionStats = nil;
  _MTEConnectionStatus: TMTEConnectionStatus = nil;
  _MTEGetVersion: TMTEGetVersion = nil;
  _MTEConnectionCerts: TMTEConneectionCerts = nil;
  _MTEGetServInfo: TMTEGetServInfo = nil;
  _MTESoftConnect: TMTESoftConnect = nil;
  _MTELogon: TMTELogon = nil;
  _MTESelectBoards: TMTESelectBoards = nil;

  MTESrlLoaded: Boolean = False;
  MTESrlAddons: TMTESrlAddOns = [];

const
  micexdll = 'mtesrl.dll';

function MTEConnect; external micexdll;
function MTEDisconnect; external micexdll;
function MTEStructure; external micexdll;
function MTEExecTrans; external micexdll;
function MTEOpenTable; external micexdll;
function MTECloseTable; external micexdll;
function MTEAddTable; external micexdll;
function MTERefresh; external micexdll;
function MTEErrorMsg; external micexdll;
function MTEFreeBuffer; external micexdll;
function MTEGetSnapshot; external micexdll;
function MTESetSnapshot; external micexdll;

resourcestring
  rsNoExecTransIP = 'MTEExecTransIP() function is not supported by MTESRL.DLL library';
  rsNoGetExtData  = 'MTEGetExtData() function is not supported by MTESRL.DLL library';
  rsNoGetServInfo = 'MTEGetServInfo() function is not supported by MTESRL.DLL library';
  rsNoSoftConnect = 'MTESoftConnect() function is not supported by MTESRL.DLL library';

procedure LoadMTESrl;
var
  H: THandle;
begin
  if not MTESrlLoaded then
  begin
    MTESrlLoaded := True;
    H := GetModuleHandle(micexdll);
    if H <> 0 then
    begin
      _MTEExecTransIP := GetProcAddress(H, 'MTEExecTransIP');
      _MTEGetExtData := GetProcAddress(H, 'MTEGetExtData');
      _MTEConnectionStats := GetProcAddress(H, 'MTEConnectionStats');
      _MTEConnectionStatus := GetProcAddress(H, 'MTEConnectionStatus');
      _MTEGetVersion := GetProcAddress(H, 'MTEGetVersion');
      _MTEConnectionCerts := GetProcAddress(H, 'MTEConnectionCerts');
      _MTEGetServInfo := GetProcAddress(H, 'MTEGetServInfo');
      _MTESoftConnect := GetProcAddress(H, 'MTESoftConnect');
      _MTELogon := GetProcAddress(H, 'MTELogon');
      _MTESelectBoards := GetProcAddress(H, 'MTESelectBoards');

      if Assigned(_MTEExecTransIP) then Include(MTESrlAddons, msaExecTransIP);
      if Assigned(_MTEGetExtData) then Include(MTESrlAddons, msaGetExtData);
      if Assigned(_MTEConnectionStats) then Include(MTESrlAddons, msaConnStats);
      if Assigned(_MTEConnectionStatus) then Include(MTESrlAddons, msaConnStatus);
      if Assigned(_MTEGetVersion) then Include(MTESrlAddons, msaGetVersion);
      if Assigned(_MTEConnectionCerts) then Include(MTESrlAddons, msaConnCerts);
      if Assigned(_MTEGetServInfo) then Include(MTESrlAddons, msaGetServInfo);
      if Assigned(_MTESoftConnect) then Include(MTESrlAddons, msaSoftConnect);
      if Assigned(_MTELogon) then Include(MTESrlAddons, msaLogon);
      if Assigned(_MTESelectBoards) then Include(MTESrlAddons, msaSelectBoards);
    end;
  end;
end;

function MTEExecTransIP(Idx: Integer; TransName, Params, ResultMsg: LPSTR; ClientIP: Integer): Integer;
begin
  LoadMTESrl;
  if Assigned(_MTEExecTransIP) then
    Result := _MTEExecTransIP(Idx, TransName, Params, ResultMsg, ClientIp)
  else
  begin
    Result := MTE_TRANSREJECTED;
    lstrcpyn(ResultMsg, PChar(rsNoExecTransIP), 256);
  end;
end;

var
  TmpBuf: array [0..255] of Char;

function MTEGetExtData(Idx: Integer; DataSetName, ExtFileName: LPSTR; var Msg: PMTEMsg): Integer; stdcall;
begin
  LoadMTESrl;
  if Assigned(_MTEGetExtData) then
    Result := _MTEGetExtData(Idx, DataSetName, ExtFileName, Msg)
  else
  begin
    Result := MTE_TRANSREJECTED;
    PInteger(@TmpBuf)^ := Length(rsNoGetExtData);
    lstrcpyn(TmpBuf + SizeOf(Integer), PChar(rsNoGetExtData), SizeOf(TmpBuf) - SizeOf(Integer));
    Msg := @TmpBuf;
  end;
end;

function MTEConnectionStats(Idx: Integer; var Stats: TMTEConnStats): Integer; stdcall;
begin
  LoadMTESrl;
  if Assigned(_MTEConnectionStats) then
    Result := _MTEConnectionStats(Idx, Stats)
  else
  begin
    Result := MTE_NOTIMPLEMENTED;
  end;
end;

function MTEConnectionStatus(Idx: Integer): Integer; stdcall;
begin
  LoadMTESrl;
  if Assigned(_MTEConnectionStatus) then
    Result := _MTEConnectionStatus(Idx)
  else
  begin
    Result := MTE_NOTIMPLEMENTED;
  end;
end;

const
  NullChar = #0;
  
function MTEGetVersion: PChar; stdcall;
begin
  LoadMTESrl;
  if Assigned(_MTEGetVersion) then
    Result := _MTEGetVersion
  else
  begin
    Result := NullChar;
  end;
end;

function MTEConnectionCerts(Idx: Integer; MyCert, ServerCert: PMTEConnCertificate): Integer; stdcall;
begin
  LoadMTESrl;
  if Assigned(_MTEConnectionCerts) then
    Result := _MTEConnectionCerts(Idx, MyCert, ServerCert)
  else
  begin
    Result := MTE_NOTIMPLEMENTED;
  end;
end;

function MTEGetServInfo(Idx: Integer; var ServInfo: PChar; var Len: Integer): Integer; stdcall;
begin
  LoadMTESrl;
  if Assigned(_MTEGetServInfo) then
    Result := _MTEGetServInfo(Idx, ServInfo, Len)
  else
  begin
    Result := MTE_NOTIMPLEMENTED;
    ServInfo := PChar(rsNoGetServInfo);
    Len := Length(rsNoGetServInfo);
  end;
end;

function MTESoftConnect(Params, ErrorMsg: LPSTR): Integer; stdcall;
begin
  LoadMTESrl;
  if Assigned(_MTESoftConnect) then
    Result := _MTESoftConnect(Params, ErrorMsg)
  else
  begin
    Result := MTE_CONFIG;
    lstrcpyn(ErrorMsg, PChar(rsNoSoftConnect), 256);
  end;
end;

function MTELogon(Idx: Integer; sync_time: Integer): Integer; stdcall;
begin
  LoadMTESrl;
  if Assigned(_MTELogon) then
    Result := _MTELogon(Idx, sync_time)
  else
    Result := MTE_NOTIMPLEMENTED;
end;

function MTESelectBoards(Idx: Integer; BoardList: LPSTR): Integer; stdcall;
begin
  LoadMTESrl;
  if Assigned(_MTESelectBoards) then
    Result := _MTESelectBoards(Idx, BoardList)
  else
    Result := MTE_NOTIMPLEMENTED;
end;

function GetMTESrlAddons: TMTESrlAddOns;
begin
  LoadMTESrl;
  Result := MTESrlAddons;
end;

end.
